@@ -5,17 +5,22 @@
 
 from tornado import web
 
-from ...base.handlers import APIHandler, json_errors
+from ...base.handlers import APIHandler
 from . import csp_report_uri
 
 class CSPReportHandler(APIHandler):
     '''Accepts a content security policy violation report'''
 
-    def skip_origin_check(self):
+    _track_activity = False
+
+    def skip_check_origin(self):
         """Don't check origin when reporting origin-check violations!"""
         return True
 
-    @json_errors
+    def check_xsrf_cookie(self):
+        # don't check XSRF for CSP reports
+        return
+
     @web.authenticated
     def post(self):
         '''Log a content security policy violation report'''
