@@ -4,31 +4,83 @@
 define([
     'jquery',
     'base/js/utils',
+    'base/js/i18n',
     'notebook/js/cell',
-    'base/js/security',
+    'base/js/markdown',
     'services/config',
-    'notebook/js/mathjaxutils',
     'notebook/js/celltoolbar',
-    'components/marked/lib/marked',
     'codemirror/lib/codemirror',
     'codemirror/mode/gfm/gfm',
-    'notebook/js/codemirror-ipythongfm'
+    'notebook/js/codemirror-ipythongfm',
+    'bidi/bidi'
 ], function(
     $,
     utils,
+    i18n,
     cell,
-    security,
+    markdown,
     configmod,
-    mathjaxutils,
     celltoolbar,
-    marked,
     CodeMirror,
     gfm,
-    ipgfm
+    ipgfm,
+    bidi
     ) {
     "use strict";
     function encodeURIandParens(uri){return encodeURI(uri).replace('(','%28').replace(')','%29')}
 
+    /**
+     * Given a file name and a list of existing file names, returns a new file name
+     * that is not in the existing list. If the file name already exists, a new one with
+     * an incremented index is returned instead.
+     *
+     * Example:
+     *  addIndexToFileName('attachment.png',
+     *                    ['attachment.png', 'attachment-3.png']) returns 'attachment-4.png'
+     *
+     * @param  {string} fileName  - original file name
+     * @param  {string} fileNames - other file names
+     * @return {string}             the original file name or one with a postfix
+     *                              index (before the extension, if one exists)
+     */
+    function addIndexToFileName(fileName, fileNames) {
+        if (fileNames === undefined) {
+            return fileName;
+        }
+        var lastDot = fileName.lastIndexOf('.');
+        var pre = fileName.substr(0, lastDot);
+        var optionalExt = fileName.substr(lastDot);
+
+        var indexMatch = '-(\\d+)';
+        // Make the index match optional so we can match both 'fileName.png' and 'fileName-2.png'
+        // The ?: makes it a non-capturing group.
+        var optionalIndexMatch = '(?:' + indexMatch + ')?';
+
+        var regex = new RegExp(pre + optionalIndexMatch + optionalExt);
+
+        var highestIndex = 0;
+        for (var existingFileName in fileNames) {
+            var match = existingFileName.match(regex);
+            var index = match[1];
+            if (index === undefined) {
+                index = 1;
+            }
+            else {
+                index = parseInt(index);
+            }
+            if (index > highestIndex) {
+                highestIndex = index;
+            }
+        }
+
+        if (highestIndex > 0) {
+            return pre + "-" + (highestIndex + 1) + optionalExt;
+        }
+        else {
+            return fileName;
+        }
+    };
+
     var Cell = cell.Cell;
 
     var TextCell = function (options) {
@@ -55,8 +107,7 @@ define([
         this.notebook = options.notebook;
         this.events = options.events;
         this.config = options.config;
-        this.notebook = options.notebook;
-        
+
         // we cannot put this as a class key as it has handle to "this".
         Cell.apply(this, [{
                     config: options.config, 
@@ -64,7 +115,6 @@ define([
                     events: this.events}]);
 
         this.cell_type = this.cell_type || 'text';
-        mathjaxutils = mathjaxutils;
         this.rendered = false;
     };
 
@@ -97,7 +147,7 @@ define([
             cell: this, 
             notebook: this.notebook});
         inner_cell.append(this.celltoolbar.element);
-        var input_area = $('<div/>').addClass('input_area');
+        var input_area = $('<div/>').addClass('input_area').attr("aria-label", i18n.msg._("Edit Markup Text here"));
         this.code_mirror = new CodeMirror(input_area.get(0), this._options.cm_config);
         // In case of bugs that put the keyboard manager into an inconsistent state,
         // ensure KM is enabled when CodeMirror is focused:
@@ -157,7 +207,7 @@ define([
     /**
      * setter: {{#crossLink "TextCell/set_text"}}{{/crossLink}}
      * @method get_text
-     * @retrun {string} CodeMirror current text value
+     * @return {string} CodeMirror current text value
      */
     TextCell.prototype.get_text = function() {
         return this.code_mirror.getValue();
@@ -246,9 +296,9 @@ define([
                 // can reference an image in markdown (using []() or a
                 // HTML <img>)
                 var text = this.get_text();
-                marked(text, function (err, html) {
-                    html = security.sanitize_html(html);
-                    html = $($.parseHTML(html));
+                markdown.render(text, {
+                    sanitize: true,
+                }, function (err, html) {
                     html.find('img[src^="attachment:"]').each(function (i, h) {
                         h = $(h);
                         var key = h.attr('src').replace(/^attachment:/, '');
@@ -300,7 +350,7 @@ define([
 
     MarkdownCell.options_default = {
         cm_config: {
-            mode: 'ipythongfm'
+            mode: 'ipythongfm',
         },
         placeholder: "Type *Markdown* and LaTeX: $\\alpha^2$"
     };
@@ -355,18 +405,37 @@ define([
         var key;
         if (blob.name !== undefined) {
             key = encodeURIandParens(blob.name);
+
+            // Add an index to the filename if we already have one with the same name
+            key = addIndexToFileName(key, that.attachments);
         } else {
             key = '_auto_' + Object.keys(that.attachments).length;
         }
 
         reader.onloadend = function() {
             var d = utils.parse_b64_data_uri(reader.result);
+            var blobData = d[1]
+
             if (blob.type != d[0]) {
                 // TODO(julienr): Not sure what we should do in this case
                 console.log('File type (' + blob.type + ') != data-uri ' +
                             'type (' + d[0] + ')');
             }
-            that.add_attachment(key, blob.type, d[1]);
+
+            // If we have the same attachment already under another key, we change the key to that.
+            // This ensures we don't create two attachments if pasting the same image twice.
+
+            for (var savedKey in that.attachments) {
+                var attachment = that.attachments[savedKey];
+                if (attachment === undefined) continue;
+
+                var savedBlob = attachment[blob.type];
+                if (savedBlob === blobData) {
+                    key = savedKey;
+                }
+            }
+
+            that.add_attachment(key, blob.type, blobData);
             var img_md = '![' + key + '](attachment:' + key + ')';
             that.code_mirror.replaceRange(img_md, pos);
         }
@@ -388,13 +457,11 @@ define([
             var text = this.get_text();
             var math = null;
             if (text === "") { text = this.placeholder; }
-            var text_and_math = mathjaxutils.remove_math(text);
-            text = text_and_math[0];
-            math = text_and_math[1];
-            marked(text, function (err, html) {
-                html = mathjaxutils.replace_math(html, math);
-                html = security.sanitize_html(html);
-                html = $($.parseHTML(html));
+            markdown.render(text, {
+                with_math: true,
+                clean_tables: true,
+                sanitize: true,
+            }, function (err, html) {
                 // add anchors to headings
                 html.find(":header").addBack(":header").each(function (i, h) {
                     h = $(h);
@@ -454,7 +521,7 @@ define([
         $(div).on('paste', function(evt) {
             var data = evt.originalEvent.clipboardData;
             var items = data.items;
-            if (data.items !== undefined) {
+            if (items !== undefined) {
                 for (var i = 0; i < items.length; ++i) {
                     var item = items[i];
                     if (item.kind == 'file' && attachment_regex.test(item.type)) {
@@ -479,9 +546,9 @@ define([
 
         // We want to display a visual indicator that the drop is possible.
         // The dragleave event is fired when we hover a child element (which
-        // is often immediatly after we got the dragenter), so we keep track
+        // is often immediately after we got the dragenter), so we keep track
         // of the number of dragenter/dragleave we got, as discussed here :
-        // http://stackoverflow.com/q/7110353/116067
+        // https://stackoverflow.com/q/7110353/116067
         // This doesn't seem to be 100% reliable, so we clear the dropzone
         // class when the cell is rendered as well
         this.code_mirror.on("dragenter", function(cm, evt) {
@@ -546,9 +613,9 @@ define([
         highlight_modes : {
             'diff'         :{'reg':[/^diff/]}
         },
-        placeholder : "Write raw LaTeX or other formats here, for use with nbconvert. " +
+        placeholder : i18n.msg._("Write raw LaTeX or other formats here, for use with nbconvert. " +
             "It will not be rendered in the notebook. " +
-            "When passing through nbconvert, a Raw Cell's content is added to the output unmodified.",
+            "When passing through nbconvert, a Raw Cell's content is added to the output unmodified."),
     };
 
     RawCell.prototype = Object.create(TextCell.prototype);
